#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include "globals.h"
#include "color_mode.h"
#include "sprite.h"
#include "image_data.h"


#define DEFAULT_FIFO_SIZE (256 * 1024)


int main (int argc, char **argv)
{
  GXColor background = {0, 0, 0, 0xff} ;
  void *gp_fifo = NULL;
  struct ir_t wiimote0_ir ;
  struct sprite *sprite ;

  /* Initialize the Wii. */
  VIDEO_Init () ;
  WPAD_Init () ;

  /* Say we want to get buttons, acceleromater and IR-tracking info. */
  WPAD_SetDataFormat (WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR) ;

  //  rmode = VIDEO_GetPreferredMode (NULL) ;
  switch (VIDEO_GetCurrentTvMode ()) {
  case VI_NTSC: rmode = &TVNtsc480IntDf ; break ;
  case VI_PAL: rmode = &TVPal528IntDf ; break ;
  case VI_MPAL: rmode = &TVMpal480IntDf ; break ;
  default: rmode = &TVNtsc480IntDf ; break ;
  }

  /* Allocate 1 framebuffer. */
  frameBuffer = MEM_K0_TO_K1 (SYS_AllocateFramebuffer (rmode)) ;

  VIDEO_Configure(rmode) ;

  /* Switch to initial framebuffer. */
  VIDEO_SetNextFramebuffer (frameBuffer) ;
  VIDEO_SetBlack (FALSE) ;                   /* Make the display visible. */
  VIDEO_Flush () ;
  VIDEO_WaitVSync () ;                /* Wait for the next vertical retrace. */
  if (rmode->viTVMode & VI_NON_INTERLACE) VIDEO_WaitVSync () ;

  /* Setup the fifo and then init the flipper. */
  gp_fifo = memalign (32, DEFAULT_FIFO_SIZE) ;
  memset (gp_fifo, 0, DEFAULT_FIFO_SIZE) ;
  GX_Init (gp_fifo, DEFAULT_FIFO_SIZE) ;

  /* Set the color to clear the bg and value to clear the z buffer. */
  GX_SetCopyClear (background, GX_MAX_Z24) ;
  VIDEO_ClearFrameBuffer (rmode, frameBuffer, 0x00800080) ;

  /* Copy the EFB to XFB. Flag tells if the color and Z buffers will */
  /* be cleared during the copy using values of GX_SetCopyClear).    */
  GX_CopyDisp (frameBuffer, GX_TRUE) ;

  sprite =
    alloc_sprite
    (SPRITE_WIDTH, SPRITE_HEIGHT, (u32*) sprite_data, (u8*) sprite_mask) ;
  if (sprite == NULL) exit (0) ;

  WPAD_IR (WPAD_CHAN_0, &wiimote0_ir) ;
  move_sprite (sprite, (s16) wiimote0_ir.x, (s16) wiimote0_ir.y) ;
  show_sprite (sprite) ;

  for (;;) {
    /* Ask info about the position of the wiimote. */
    WPAD_IR (WPAD_CHAN_0, &wiimote0_ir) ;

    /* Do this before drawing. */
    GX_SetViewport (0, 0, rmode->fbWidth, rmode->efbHeight, 0, 1) ;

    move_sprite (sprite, (s16) wiimote0_ir.x, (s16) wiimote0_ir.y) ;

    /* Do this stuff after drawing. */
    GX_DrawDone () ;
    VIDEO_Flush () ;
    VIDEO_WaitVSync () ;

    /* Exit if requested. */
    WPAD_ScanPads () ;
    if (WPAD_ButtonsDown (WPAD_CHAN_0) & WPAD_BUTTON_HOME) {
      hide_sprite (sprite) ;
      free_sprite (sprite) ;
      exit(0) ;
    }
  }

  return (0) ;
}
